package com.huiju.welcome.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyuncs.CommonRequest;
import com.aliyuncs.CommonResponse;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.exceptions.ServerException;
import com.aliyuncs.green.model.v20180509.AddFacesRequest;
import com.aliyuncs.green.model.v20180509.AddPersonRequest;
import com.aliyuncs.green.model.v20180509.GetPersonRequest;
import com.aliyuncs.green.model.v20180509.ImageSyncScanRequest;
import com.aliyuncs.http.FormatType;
import com.aliyuncs.http.HttpResponse;
import com.aliyuncs.http.MethodType;
import com.aliyuncs.profile.DefaultProfile;
import com.aliyuncs.profile.IClientProfile;
import com.huiju.welcome.config.AliProperties;
import com.tencentcloudapi.common.exception.TencentCloudSDKException;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.web.client.RestTemplate;

import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

@Slf4j
@Configuration
@EnableConfigurationProperties(AliProperties.class)
public class AliFaceUtils {

    public final static String ACT_ADD_FACE = "AddFace";
    public final static String ACT_RECOGNIZE_FACE = "RecognizeFace";

    DefaultProfile profile;
    DefaultAcsClient client;
    IAcsClient recognitionClient;

    @Autowired
    public AliFaceUtils(AliProperties p) throws ClientException {
        profile = DefaultProfile.getProfile(AliProperties.face.getRegionId(), AliProperties.accessKeyId, AliProperties.accessKeySecret);
        client = new DefaultAcsClient(profile);
        IClientProfile iClientProfile = DefaultProfile.getProfile(AliProperties.face.getRegionId(), AliProperties.accessKeyId, AliProperties.accessKeySecret);
        recognitionClient = new DefaultAcsClient(iClientProfile);
    }

    /**
     * 检测照片中是否有人脸
     * @param image
     * @return
     */
    public boolean detectFace(String image)  {
        if (null == image) return false;

        log.info("阿里云开始检测是否有人脸");
        log.info("人脸图 {}", image);

        String url = "https://dtplus-cn-shanghai.data.aliyuncs.com/face/detect";
        Date now = new Date();
        String gmtStr = DateUtils.toGMTString(now);

        Map<String, Object> sendData = new HashMap<String, Object>();
        sendData.put("type", 0);
        sendData.put("image_url", image);
        String sendJSON = JSONObject.toJSONString(sendData);

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.set("accept", MediaType.APPLICATION_JSON_VALUE);
        headers.set("date", gmtStr);
        headers.set("Authorization", "Dataplus "+ getSignature(url, "POST", MediaType.APPLICATION_JSON_VALUE, gmtStr, sendJSON));

        RestTemplate restTemplate = new RestTemplate();
        HttpEntity<String> request = new HttpEntity<>(sendJSON, headers);
        String resp ="";
        try{
            resp = restTemplate.postForEntity(url, request, String.class).getBody();

            JSONObject result = JSONObject.parseObject(resp);

            Integer faceNum = result.getInteger("face_num");

            log.info("人脸数目: {}", faceNum);

            return null != faceNum && faceNum > 0;
        }catch (Exception e){
            log.error("检测人脸失败: {}", e.getMessage());
            e.printStackTrace();
        }

        log.info("未检测到人脸");
        return false;
    }
    
    /**
     * 获取用户信息
     * @param personId
     * @return
     * @throws UnsupportedEncodingException
     */
    public JSONObject getPersonBaseInfo(String personId) throws Exception {
        GetPersonRequest getPersonRequest = new GetPersonRequest();
        getPersonRequest.setAcceptFormat(FormatType.JSON); // 指定api返回格式
        getPersonRequest.setMethod(com.aliyuncs.http.MethodType.POST); // 指定请求方法
        getPersonRequest.setEncoding("utf-8");
    
        JSONObject data = new JSONObject();
        /**
         * personId: 用户自定义个体Id，必填
         */
        data.put("personId", personId);
    
    
        getPersonRequest.setHttpContent(data.toJSONString().getBytes("UTF-8"), "UTF-8", FormatType.JSON);
    
        /**
         * 请务必设置超时时间
         */
        getPersonRequest.setConnectTimeout(3000);
        getPersonRequest.setReadTimeout(6000);
    
        try {
            HttpResponse httpResponse = recognitionClient.doAction(getPersonRequest);
            if (httpResponse.isSuccess()) {
                JSONObject scrResponse = JSON.parseObject(new String(httpResponse.getHttpContent(), "UTF-8"));
                if (200 == scrResponse.getInteger("code")) {
                    JSONObject resultObject = scrResponse.getJSONObject("data");
                    if (200 == resultObject.getInteger("code")) {
                        return resultObject;
                    } else {
                        throw new Exception("task process fail:" + resultObject.getInteger("code"));
                    }
                } else if(400 == scrResponse.getInteger("code")){
                    return null;
                }else{
                    throw new Exception("detect not success. code:" + scrResponse.getInteger("code"));
                }
            }else{
                throw new Exception("response not success. status:" + httpResponse.getStatus());
            }
        } catch (Exception e) {
            throw e;
        }
    }
    
    /**
     * 创建人员
     * @param personId
     * @param personName
     * @throws TencentCloudSDKException
     * @throws UnsupportedEncodingException
     */
    public JSONObject createPerson(String personId, String personName) throws Exception {
        // 已经存在的人员, 不重复创建
        try {
            if (null != getPersonBaseInfo(personId)){
                return getPersonBaseInfo(personId);
            }
        } catch (Exception e) {
           throw e;
        }

        AddPersonRequest addPersonRequest = new AddPersonRequest();
        // 指定api返回格式
        addPersonRequest.setAcceptFormat(FormatType.JSON);
        // 指定请求方法
        addPersonRequest.setMethod(com.aliyuncs.http.MethodType.POST);
        addPersonRequest.setEncoding("utf-8");
    
        JSONObject data = new JSONObject();
        /**
         * personId: 用户自定义个体Id，必填
         * groupIds: 用户自定义组Id列表，必填
         * name: 用户名称，非必填
         * note: 备注信息，非必填
         */
        data.put("personId", personId);
        data.put("groupIds", Arrays.asList(AliProperties.face.getGroup()));
        data.put("name", personName);
        data.put("note", "备注信息");
    
        addPersonRequest.setHttpContent(data.toJSONString().getBytes("UTF-8"), "UTF-8", FormatType.JSON);
    
        /**
         * 请务必设置超时时间
         */
        addPersonRequest.setConnectTimeout(3000);
        addPersonRequest.setReadTimeout(6000);
    
        try {
            HttpResponse httpResponse = recognitionClient.doAction(addPersonRequest);
        
            if (httpResponse.isSuccess()) {
                JSONObject scrResponse = JSON.parseObject(new String(httpResponse.getHttpContent(), "UTF-8"));
                if (200 == scrResponse.getInteger("code")) {
                    JSONObject resultObject = scrResponse.getJSONObject("data");
                    if (200 == resultObject.getInteger("code")) {
                        return resultObject;
                    } else {
                        throw new Exception("task process fail:" + resultObject.getInteger("code"));
                    }
                } else {
                    throw new Exception("detect not success. code:" + scrResponse.getInteger("code"));
                }
            } else {
                throw new Exception("response not success. status:" + httpResponse.getStatus());
            }
        } catch (Exception e) {
            throw e;
        }
    }

    /**
     * 创建人员
     * @param personId
     * @param url
     * @return
     * @throws Exception
     */
    public JSONObject createFace(String personId, String url) throws Exception {
        JSONObject result = new JSONObject();
        try {
            createPerson(personId, personId);
        } catch (Exception e) {
            throw e;
        }
    
        AddFacesRequest addFacesRequest = new AddFacesRequest();
        addFacesRequest.setAcceptFormat(FormatType.JSON); // 指定api返回格式
        addFacesRequest.setMethod(com.aliyuncs.http.MethodType.POST); // 指定请求方法
        addFacesRequest.setEncoding("utf-8");
    
        JSONObject data = new JSONObject();
        /**
         * personId: 用户自定义个体Id，必填
         * urls: 图片url
         */
        data.put("personId", personId);
        data.put("urls", Arrays.asList(url));
    
        addFacesRequest.setHttpContent(data.toJSONString().getBytes("UTF-8"), "UTF-8", FormatType.JSON);
    
        /**
         * 请务必设置超时时间
         */
        addFacesRequest.setConnectTimeout(3000);
        addFacesRequest.setReadTimeout(6000);
    
        try {
            HttpResponse httpResponse = recognitionClient.doAction(addFacesRequest);
        
            if (httpResponse.isSuccess()) {
                JSONObject scrResponse = JSON.parseObject(new String(httpResponse.getHttpContent(), "UTF-8"));
                System.out.println(JSON.toJSONString(scrResponse, true));
                if (200 == scrResponse.getInteger("code")) {
                    JSONObject resultObject = scrResponse.getJSONObject("data");
                    if (200 == resultObject.getInteger("code")) {
                        System.out.println(resultObject.getString("personId"));
                    } else {
                        System.out.println("task process fail:" + resultObject.getInteger("code"));
                    }
                } else {
                    System.out.println("detect not success. code:" + scrResponse.getInteger("code"));
                }
            } else {
                System.out.println("response not success. status:" + httpResponse.getStatus());
            }
        } catch (ServerException e) {
            e.printStackTrace();
        } catch (ClientException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 人脸查询
     * @param url
     * @return
     * @throws Exception
     */
    public JSONObject searchFace(String url) throws Exception {
        ImageSyncScanRequest imageSyncScanRequest = new ImageSyncScanRequest();
        imageSyncScanRequest.setAcceptFormat(FormatType.JSON); // 指定api返回格式
        imageSyncScanRequest.setMethod(com.aliyuncs.http.MethodType.POST); // 指定请求方法
        imageSyncScanRequest.setEncoding("utf-8");
        imageSyncScanRequest.setRegionId(AliProperties.face.getRegionId());
    
    
        List<Map<String, Object>> tasks = new ArrayList<Map<String, Object>>();
        Map<String, Object> task = new LinkedHashMap<String, Object>();
        task.put("dataId", UUID.randomUUID().toString());
        task.put("url", url);
        task.put("time", new Date());
        Map<String, String> extras = new HashMap<String, String>();
        extras.put("groupId", AliProperties.face.getGroup());
        task.put("extras", extras);
    
        tasks.add(task);
        JSONObject data = new JSONObject();
        /**
         * sface-n: 人脸1-N
         */
        data.put("scenes", Arrays.asList("sface-n"));
        data.put("tasks", tasks);
    
        imageSyncScanRequest.setHttpContent(data.toJSONString().getBytes("UTF-8"), "UTF-8", FormatType.JSON);
    
        /**
         * 请务必设置超时时间
         */
        imageSyncScanRequest.setConnectTimeout(3000);
        imageSyncScanRequest.setReadTimeout(6000);
    
        try {
            HttpResponse httpResponse = client.doAction(imageSyncScanRequest);
        
            if (httpResponse.isSuccess()) {
                JSONObject scrResponse = JSON.parseObject(new String(httpResponse.getHttpContent(), "UTF-8"));
                System.out.println(JSON.toJSONString(scrResponse, true));
                if (200 == scrResponse.getInteger("code")) {
                    JSONArray taskResults = scrResponse.getJSONArray("data");
                    for (Object taskResult : taskResults) {
                        if(200 == ((JSONObject)taskResult).getInteger("code")){
                            JSONArray sceneResults = ((JSONObject)taskResult).getJSONArray("results");
                            for (Object sceneResult : sceneResults) {
                                JSONArray personList = ((JSONObject) sceneResult).getJSONArray("topPersonData");
                                log.info("查询阿里云人脸结果: {}", personList);

                                if (null != personList && personList.size() > 0) {
                                    JSONArray persons = personList.getJSONObject(0).getJSONArray("persons");
                                    if (null != persons && persons.size() > 0) {
                                        JSONObject person = persons.getJSONObject(0);
                                        if (null != person.getString("personId") && !"null".equals(person.getString("personId"))) {
                                            return person;
                                        } else if (persons.size() > 1) {
                                            return persons.getJSONObject(1);
                                        }
                                    }
                                }
                            }
                        }else{
                            throw new Exception("task process fail:" + ((JSONObject)taskResult).getInteger("code"));
                        }
                    }
                } else {
                    throw new Exception("detect not success. code:" + scrResponse.getInteger("code"));
                }
            } else {
                throw new Exception("response not success. status:" + httpResponse.getStatus());
            }
        } catch (Exception e){
            throw e;
        }
        return null;
    }

    /**
     * 添加人脸
     * @param personId
     * @param name
     * @param url
     * @param datas
     * @throws Exception
     */
    public void addFace(Integer personId, String name, String url, String ...datas) throws Exception {
        log.info("阿里人脸比对, 请求上传人脸: {}[{}] {}", name, personId, url);

        if (null == url && null == datas) {
            throw new Exception("图片地址 或者 Base64 数据必填一个");
        }

        CommonRequest request = generalRequest(ACT_ADD_FACE);
        request.putBodyParameter("Person", name);
        request.putBodyParameter("Image", personId);

        if (null == datas || datas.length < 1) {
            request.putBodyParameter("ImageUrl", url);
        } else {
            request.putBodyParameter("Content", datas[0]);
        }

        excuteRequest(request);
    }

    /**
     * 人脸库比对
     * @param url
     * @param datas
     * @throws Exception
     */
    public JSONObject recognizeFace(String url, String ...datas) throws Exception {
        log.info("阿里人脸比对, 请求比对人脸: {}", url);

        if (null == url && null == datas) {
            throw new Exception("图片地址 或者 Base64 数据必填一个");
        }

        CommonRequest request = generalRequest(ACT_RECOGNIZE_FACE);

        if (null == datas || datas.length < 1) {
            request.putBodyParameter("ImageUrl", url);
        } else {
            request.putBodyParameter("Content", datas[0]);
        }

        return excuteRequest(request);
    }

    private JSONObject excuteRequest(CommonRequest request) throws Exception {
        CommonResponse response = client.getCommonResponse(request);
        if (response.getHttpStatus() == HttpStatus.SC_OK) {
            try {
                JSONObject result = JSONObject.parseObject(response.getData());

                Boolean success = result.getBoolean("Success");
                if (null == success || !success) {
                    String error = "校验阿里人脸比对接口返回出错: " + result.getString("Message");
                    log.error(error);
                    throw new Exception(error);
                }

                return result;
            } catch (Exception e) {
                String error = "校验阿里人脸比对接口出错: " + e.getMessage();
                log.error(error);
                e.printStackTrace();
                throw new Exception(error);
            }
        } else {
            String error = "网络请求上传比对人脸失败, 错误码: {}" + response.getHttpStatus();
            log.error(error);
            throw new Exception(error);
        }
    }

    private CommonRequest generalRequest(String action) {
        CommonRequest request = new CommonRequest();
        request.setMethod(MethodType.POST);
        request.setDomain(AliProperties.face.getDomain());
        request.setVersion(AliProperties.face.getVersion());
        request.setAction(action);
        request.putBodyParameter("Group", AliProperties.face.getGroup());
        return request;
    }

    private String getSignature(String url, String method, String contentType, String gmtStr, String data) {
        URL realURL = null;
        try {
            realURL = new URL(url);
        } catch (MalformedURLException e) {
            log.error("解析 URL 失败: {}", e.getMessage());
            e.printStackTrace();
            return null;
        }
        String path = realURL.getFile();

        String[] parts = new String[]{
                method,
                contentType,
                Encrypt.MD5Base64(data),
                contentType,
                gmtStr,
                path
        };

        String str2Sign = String.join("\n", parts);
        log.info("---string to sign ===> {}", str2Sign);

        String signature = AliProperties.accessKeyId + ":" + Encrypt.HMACSha1(str2Sign, AliProperties.accessKeySecret);
        log.info("---signature ===> {}", signature);
        return signature;
    }
}
