package com.huiju.welcome.utils;

import com.huiju.welcome.config.TencentProperties;
import com.tencentcloudapi.common.Credential;
import com.tencentcloudapi.common.exception.TencentCloudSDKException;
import com.tencentcloudapi.iai.v20180301.IaiClient;
import com.tencentcloudapi.iai.v20180301.models.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Configuration;

@Slf4j
@Configuration
@EnableConfigurationProperties(TencentProperties.class)
public class TencentFaceUtils {
    TencentProperties properties;
    Credential cred;

    @Autowired
    public TencentFaceUtils(TencentProperties properties) {
        this.properties = properties;
        this.cred = new Credential(properties.getSecretId(), properties.getSecretKey());
    }

    public GetPersonBaseInfoResponse getPersonBaseInfo(String personId) throws TencentCloudSDKException {
        GetPersonBaseInfoRequest request = new GetPersonBaseInfoRequest();
        request.setPersonId(personId);

        IaiClient client = new IaiClient(this.cred, this.properties.getIai().getRegion());
        return client.GetPersonBaseInfo(request);
    }

    public void createPerson(String personId, String personName, String url, String ...images) throws TencentCloudSDKException {
        // 已经存在的人员, 不重复创建
        try {
            getPersonBaseInfo(personId);
            return;
        } catch (TencentCloudSDKException e) {
            if (e.toString().indexOf("InvalidParameterValue.PersonIdNotExist") == -1) {
                throw e;
            }
        }

        CreatePersonRequest request = new CreatePersonRequest();
        request.setGroupId(this.properties.getIai().getGroup());
        request.setPersonId(personId);
        request.setPersonName(personName);
        if (null != url && !"".equals(url)) {
            request.setUrl(url);
        } else if (null != images && images.length > 0) {
            request.setImage(images[0]);
        }

        IaiClient client = new IaiClient(this.cred, this.properties.getIai().getRegion());
        client.CreatePerson(request);
    }

    public void createFace(String personId, String url, String ...images) throws TencentCloudSDKException {
        // 因为目前腾讯接口没有办法获取确切的错误码, 所以先尝试着去新增人员
        // https://github.com/TencentCloud/tencentcloud-sdk-java/issues/22

        try {
            createPerson(personId, personId, url);
        } catch (TencentCloudSDKException e) {
            log.error(e.toString());
        }

        CreateFaceRequest request = new CreateFaceRequest();
        request.setPersonId(personId);
        if (null != url && !"".equals(url)) {
            request.setUrls(new String[]{ url });
        } else if (null != images && images.length > 0) {
            request.setImages(images);
        } else {
            throw new TencentCloudSDKException("必须设置人脸图片");
        }

        IaiClient client = new IaiClient(this.cred, this.properties.getIai().getRegion());
        client.CreateFace(request);
    }

    public Candidate searchFace(String url, String ...images) throws TencentCloudSDKException {
        SearchFacesRequest request = new SearchFacesRequest();
        request.setGroupIds(new String[]{ this.properties.getIai().getGroup() });
        if (null != url && !"".equals(url)) {
            request.setUrl(url);
        } else if (null != images && images.length > 0) {
            request.setImage(images[0]);
        } else {
            throw new TencentCloudSDKException("必须设置人脸图片");
        }

        IaiClient client = new IaiClient(this.cred, this.properties.getIai().getRegion());
        SearchFacesResponse response = client.SearchFaces(request);
        Result[] results = response.getResults();
        if (null == results || results.length < 1) return null;

        Candidate[] candidates = results[0].getCandidates();
        return candidates[0];
    }
}
